{% extends "base.html" %}

{% block title %}Players - StratumGS{% end %}

{% block content %}
    <h1>Players</h1>
    
    {% if players %}
        <ul class="page-list">
            {% for player in players %}
                <li>
                    <h2>{{ player.name }}</h2>
                    <p>Supported Games: {{ ", ".join(player.supported_games_display) }}</p>
                    <p>{{ player.games_available }}/{{ player.max_games }} games available.</p>
                </li>
            {% end %}
        </ul>
    {% else %}
        <p>No connected players.</p>
    {% end %}
{% end %}
