{% extends "base.html" %}

{% block title %}StratumGS{% end %}

{% block content %}
    <h1>About</h1>

    <p>StratumGS is a turn based game engine designed to pit user-designed autonomous players against each other.</p>
{% end %}
