"""
.. module stratumgs.game.engine

This module contains the base engine that all game engines extend, as well as
code for interacting with clients.
"""

# flake8:noqa

from .engine import BaseEngine
